/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef PARAMETRIC_SURFACE_HPP
#define PARAMETRIC_SURFACE_HPP


#include "mesh.hpp"

#include <vector>
#include <string>



namespace cpe
{


/** \brief Container class for a generic mesh with normal, color, texture, ... */
class parametric_surface : public mesh
{
    public:

    // ********************************************* //
    // ********************************************* //
    //  Construction
    // ********************************************* //
    // ********************************************* //


    /** \brief Resize a parametric mesh, build a default geometrical plane
     *  \param Nu: number of vertices in u direction
     *  \param Nv: number of vertices in v direction
     *
     **/
    void resize(int Nu,int Nv);

    // ********************************************* //
    // ********************************************* //
    //  Access data
    // ********************************************* //
    // ********************************************* //

    /** \brief Access to value S(u,v) */
    const vec3& operator()(int ku,int kv) const;
    /** \brief Access to value S(u,v) */
    vec3& operator()(int ku,int kv);

    /** \brief Access to color value */
    const vec3& color(int ku,int kv) const;
    /** \brief Access to color value */
    vec3& color(int ku,int kv);

    /** \brief number of samples in u direction */
    int size_u() const;
    /** \brief number of samples in v direction */
    int size_v() const;


    private:

    /** \brief Number of samples in u direction */
    int Nu;
    /** \brief Number of samples in v direction */
    int Nv;



};




}

#endif
