#include "monWidgetGL.hpp"

#include "GL/glew.h"
#include "GL/gl.h"

#include "../lib3d/matrix4.hpp"
#include "../lib3d/matrix3.hpp"

#include "../libmesh/mesh_io.hpp"
#include "maFenetre.hpp"

#include "glutils.hpp"

#include <cmath>

#include <iostream>
#include <GL/glu.h>

monWidgetGL::monWidgetGL(const QGLFormat& format,QGLWidget *parent) :
    QGLWidget(format,parent)
{
    QWidget::setFocusPolicy(Qt::StrongFocus);
    startTimer(10);
    etat_affichage=true;
}

monWidgetGL::~monWidgetGL()
{}

void monWidgetGL::initializeGL()
{
    std::cout<<"appel monWidgetGL::initializeGL()"<<std::endl;
    glewInit();
    std::cout<<"Glew initialized"<<std::endl;

    std::cout<<"appel scene::load_scene()"<<std::endl;
    scene_3d.load_scene();
    std::cout<<"scene loaded"<<std::endl;

    //activation de la gestion de la profondeur
    glEnable(GL_DEPTH_TEST);

}




void monWidgetGL::paintGL()
{
    if(nav.screen_size_x()<50)
        resize(50,nav.screen_size_y());
    if(nav.screen_size_y()<50)
        resize(nav.screen_size_x(),50);

    //setup cameras in shaders
    setup_camera();

    //clear screen
    glViewport (0, 0, nav.screen_size_x(),nav.screen_size_y());
    glClearColor(1.0, 1.0, 1.0, 1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    //draw 3D scene
    if(etat_affichage==true)
    {
        scene_3d.draw_scene();
    }
}



void monWidgetGL::afficher()
{
    etat_affichage=!etat_affichage;
std::cout<<etat_affichage<<std::endl;
    updateGL();
}
void monWidgetGL::wireframe(bool est_actif)
{
    if(est_actif==true)
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);

    updateGL();
}


void monWidgetGL::resizeGL(int width, int height)
{
    nav.screen_size_x()=width;nav.screen_size_y()=height;glViewport(0,0, width, height);
}

void monWidgetGL::mousePressEvent(QMouseEvent *event)
{
    nav.x_previous()=event->x();
    nav.y_previous()=event->y();

    updateGL();
}

void monWidgetGL::mouseMoveEvent(QMouseEvent *event)
{
    int x=event->x();
    int y=event->y();

    int ctrl_pressed = (event->modifiers() & Qt::ControlModifier);

    int shift_pressed = (event->modifiers() & Qt::ShiftModifier);
    if (!ctrl_pressed && !shift_pressed && (event->buttons() & Qt::LeftButton) )
        nav.trackball_rotate(x,y);
    else if (!ctrl_pressed && !shift_pressed && (event->buttons() & Qt::RightButton))
        nav.trackball_zoom(y);

    double dL=0.0001*(1+10*abs(nav.dist_cam()));
    if( !ctrl_pressed && shift_pressed && (event->buttons() & Qt::LeftButton) )
    {
        nav.go_up(dL*(y-nav.y_previous()));
        nav.go_right(-dL*(x-nav.x_previous()));
    }
    if( !ctrl_pressed && shift_pressed && (event->buttons() & Qt::RightButton) )
        nav.go_forward(5*dL*(y-nav.y_previous()));



    nav.x_previous()=x;
    nav.y_previous()=y;

    updateGL();

}
void monWidgetGL::timerEvent(QTimerEvent *event)
{
    event->accept();
    updateGL();
}


cpe::matrix4 build_normal_matrix(const cpe::matrix4& w)
{

    cpe::matrix4 normal_matrix=cpe::matrix4(w(0,0),w(1,0),w(2,0),0,
                                            w(0,1),w(1,1),w(2,1),0,
                                            w(0,2),w(1,2),w(2,2),0,
                                            0,0,0,1);
    cpe::vec4 tr(w(0,3),w(1,3),w(2,3),1.0f);
    cpe::vec4 tr_inv=-normal_matrix*tr;
    normal_matrix(0,3)=tr_inv[0];
    normal_matrix(1,3)=tr_inv[1];
    normal_matrix(2,3)=tr_inv[2];

    normal_matrix=normal_matrix.transposed();

    return normal_matrix;
}

void monWidgetGL::setup_camera()
{

    //**************************************************//
    // SETUP PROJECTION MATRIX
    //**************************************************//

    //parameters: left,right, bottom,top, Znear,Zfar
    std::vector<double> frustum_parameters=nav.projection_param();
    cpe::matrix4 projection_matrix=cpe::matrix4::projection_frustum(frustum_parameters[0],-frustum_parameters[0],frustum_parameters[1],-frustum_parameters[1],frustum_parameters[2],frustum_parameters[3]);
    const float* pointer_projection_matrix=projection_matrix.pointer();

    //**************************************************//
    // SETUP WORLD MATRIX (MODELVIEW)
    //**************************************************//

    cpe::matrix4 world_matrix_zoom=cpe::matrix4::translation(cpe::vec3(0,0,nav.dist_cam()));
    cpe::matrix4 world_matrix_translation=cpe::matrix4::translation(nav.camera_translation());
    cpe::matrix4 world_matrix_rotation=nav.camera_orientation();

    cpe::matrix4 world_matrix=world_matrix_zoom*world_matrix_rotation*world_matrix_translation;
    const float* pointer_world_matrix=world_matrix.pointer();

    //**************************************************//
    // SETUP NORMAL MATRIX transpose(inverse(world matrix))
    //**************************************************//

    cpe::matrix4 normal_matrix=build_normal_matrix(world_matrix);
    const float* pointer_normal_matrix=normal_matrix.pointer();


    //**************************************************//
    // SET SHADER CAMERA
    //**************************************************//

    glUniformMatrix4fv(get_uni_loc(scene_3d.get_current_shader_id(), "camera_modelview"),1,false,pointer_world_matrix);PRINT_OPENGL_ERROR();
    glUniformMatrix4fv(get_uni_loc(scene_3d.get_current_shader_id(),"camera_projection"),1,false,pointer_projection_matrix);PRINT_OPENGL_ERROR();
    glUniformMatrix4fv(get_uni_loc(scene_3d.get_current_shader_id(),"normal_matrix"),1,false,pointer_normal_matrix);PRINT_OPENGL_ERROR();

}


